/*
 * main.js
 * Describes the main logic for the application 
 */

'use strict';

//presenting the images instead of video stream
$('#remoteVideo').hide();
$('#localVideo').hide();

///////////////////////////////////////////////////////////
//variables use thorugh the code
var localuser;
var remoteuser;
var isChannelReady = false;
var isInitiator = false;
var isStarted = false;
var localStream;
var pc;
var remoteStream;
var turnReady;
var dataChannel;

//configuration of ICE server, STUN and TURN. Here uses Googels
var pcConfig = {
  'iceServers': [{
    'url': 'stun:stun.l.google.com:19302'
  }]
};

// Set up audio and video stream regardless of what devices are present.
var sdpConstraints = {
  'mandatory': {
    'OfferToReceiveAudio': true,
    'OfferToReceiveVideo': true
  }
};

//retrieves the elements from the index page
var startButton = document.getElementById("startButton");
startButton.disabled = false;
startButton.onclick = startConnection;

var localVideo = document.querySelector('#localVideo');
var remoteVideo = document.querySelector('#remoteVideo');

var room = location.pathname.substring(1);
var user = location.pathname.substring(2);

var constraints = {
  audio: true,
  video: true
};

///////////////////////////////////////////////////
//connect to the server
var socket = io.connect();

//staring the connection
function startConnection(){
   if (user === "" ){
       user = document.getElementById("userId").value;
   }
   if (room === '') {
       room = document.getElementById("roomId").value;
    }
    if(user === "" || room === ""){
        alert("Fill in Username and Room Name");
    }
    $('.navbar-toggle').trigger('click');
    localuser = document.getElementById("userId").value;
    if(user !== ""){
      socket.emit('create or join', room);
      console.log('Attempted to create or  join room', room);
   }
   navigator.getUserMedia(constraints, gotStream, handleUserMediaError);

   console.log('Getting user media with constraints', constraints);

  if (location.hostname !== 'localhost') {
    requestTurn('https://computeengineondemand.appspot.com/turn?username=41784574&key=4080218913');
  }
}

//////////////////////////////////////////
//handles the message chat from server, which presents the chat messages 
socket.on('chat', function(message){
   console.log(message);
   $('#chat').append("<span style='color:red;padding-left: 5px;'>" + message.user + "</span>: " + message.msg + "</br>");
});

//handles the chat message to send it to the other user
$('#msg').keypress(function(e){
    if(e.keyCode === 13){
        if (user === ""){
            alert('Join room first!');
            return false;
        }
     if ($('#msg').val() === '')
      return false;
    var msg = $('#msg').val();
    var msgob = {
      'user': localuser,
      'msg': msg
    };
    socket.emit('chat', msgob);
    $('#chat').append("<span style='color:green;padding-left: 5px;'>Me</span>: " + msgob.msg + "</br>");
    $('#msg').val('');
  }
});

///////////////////////////////////////////////////////////
//different functions to handle messages from the server
socket.on('created', function(room) {
  console.log('Created room ' + room);
  isInitiator = true;
});

socket.on('full', function(room) {
  console.log('Room ' + room + ' is full');
});

socket.on('join', function (room){
  console.log('Another peer made a request to join room ' + room);
  console.log('This peer is the initiator of room ' + room + '!');
  isChannelReady = true;
});

socket.on('joined', function(room) {
  console.log('joined: ' + room);
  isChannelReady = true;
});

socket.on('log', function(array) {
  console.log.apply(console, array);
});


////////////////////////////////////////////////
//sending message to the server
function sendMessage(message) {
  console.log('Client sending message: ', message);
  socket.emit('message', message);
}

//this client receives a message
socket.on('message', function(message) {
  console.log('Client received message:', message);
  if (message === 'got user media') {
    maybeStart();
  } else if (message.type === 'offer') {
    if (!isInitiator && !isStarted) {
      maybeStart();
    }
    pc.setRemoteDescription(new RTCSessionDescription(message));
    doAnswer();
  } else if (message.type === 'answer' && isStarted) {
    pc.setRemoteDescription(new RTCSessionDescription(message));
  } else if (message.type === 'candidate' && isStarted) {
    var candidate = new RTCIceCandidate({
      sdpMLineIndex: message.label,
      candidate: message.candidate
    });
    pc.addIceCandidate(candidate);
  } else if (message === 'bye' && isStarted) {
    //handleRemoteHangup();
  }
});

////////////////////////////////////////////////////
//presenting the local audio and video stream
function gotStream(stream) {
  console.log('Adding local stream.');
  localVideo.src = window.URL.createObjectURL(stream);
  localStream = stream;
  sendMessage('got user media');
  $('#localimg').hide();
  $('#localVideo').show();
  if (isInitiator) {
    maybeStart();
  }
}
//handle media error
function handleUserMediaError(error) {
  console.log('navigator.getUserMedia error: ', error);
}

//handle the creating the peer connection
function maybeStart() {
  console.log('>>> maybeStart() <<<', isStarted, localStream, isChannelReady);
  if (!isStarted && typeof localStream !== 'undefined' && isChannelReady) {
    console.log('>>creating peer connection<<');
    createPeerConnection();
    pc.addStream(localStream);
    isStarted = true;
    console.log('isInitiator', isInitiator);
    if (isInitiator) {
      doCall();
    }
  }
}

window.onbeforeunload = function() {
  sendMessage('bye');
};

/////////////////////////////////////////////////////////
//signaling messages 

//creating peer connection
function createPeerConnection() {
  try { 
    pc = new RTCPeerConnection(null); //missing something here
    pc.onicecandidate = handleIceCandidate;
    pc.onaddstream = handleRemoteStreamAdded;
    pc.onremovestream = handleRemoteStreamRemoved;
    console.log('Created RTCPeerConnnection');
  } catch (e) {
    console.log('Failed to create PeerConnection, exception: ' + e.message);
    alert('Cannot create RTCPeerConnection object.');
    return;
  }
}
//handle ICE candidate
function handleIceCandidate(event) {
  console.log('icecandidate event: ', event);
  if (event.candidate) {
    sendMessage({
      type: 'candidate',
      label: event.candidate.sdpMLineIndex,
      id: event.candidate.sdpMid,
      candidate: event.candidate.candidate
    });
  } else {
    console.log('End of candidates.');
    $('#remoteimg').hide();
    $('#remoteVideo').show();
  }
}
//handle the added remote stream
function handleRemoteStreamAdded(event) {
  console.log('Remote stream added.');
  remoteVideo.src = window.URL.createObjectURL(event.stream);
  remoteStream = event.stream;
}
//error message with creating offer message
function handleCreateOfferError(event) {
  console.log('createOffer() error: ', event);
}
//handle sending offer message
function doCall() {
  console.log('Sending offer to peer');
  pc.createOffer(setLocalAndSendMessage, handleCreateOfferError);
}
//handle sending answer message
function doAnswer() {
  console.log('Sending answer to peer.');
  pc.createAnswer().then(
    setLocalAndSendMessage,
    onCreateSessionDescriptionError
  );
}
//handle set local description
function setLocalAndSendMessage(sessionDescription) {
  pc.setLocalDescription(sessionDescription);
  console.log('setLocalAndSendMessage sending message', sessionDescription);
  sendMessage(sessionDescription);
}
//error with creating session description
function onCreateSessionDescriptionError(error) {
  trace('Failed to create session description: ' + error.toString());
}
//requesting TURN server
function requestTurn(turnURL) {
  var turnExists = false;
  for (var i in pcConfig.iceServers) {
    if (pcConfig.iceServers[i].url.substr(0, 5) === 'turn:') {
      turnExists = true;
      turnReady = true;
      break;
    }
  }
  if (!turnExists) {
    console.log('Getting TURN server from ', turnURL);
    // No TURN server, find one.
    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function() {
      if (xhr.readyState === 4 && xhr.status === 200) {
        var turnServer = JSON.parse(xhr.responseText);
        console.log('Got TURN server: ', turnServer);
        pcConfig.iceServers.push({
          'url': 'turn:' + turnServer.username + '@' + turnServer.turn,
          'credential': turnServer.password
        });
        turnReady = true;
      }
    };
    xhr.open('GET', turnURL, true);
    xhr.send();
  }
}

//handle remote stream
function handleRemoteStreamAdded(event) {
  console.log('Remote stream added.');
  remoteVideo.src = window.URL.createObjectURL(event.stream);
  remoteStream = event.stream;
}
//handle the remote stream removed
function handleRemoteStreamRemoved(event) {
  console.log('Remote stream removed. Event: ', event);
}
//stop the connection
function hangup() {
  console.log('Hanging up.');
  stop();
  sendMessage('bye');
}

function stop() {
  isStarted = false;
  pc.close();
  pc = null;
}

